package com.gx.obe.bpt.composite;

import java.util.Arrays;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class BidPriceTableItemButton extends Composite {
	
	private Composite button;
	private Composite bgComposite;
	private Label textLabel;
	
	private Optional<Color> selectBgColor = Optional.of(SWTResourceManager.getColor(190, 214, 240));
	private Optional<Color> notSelectBgColor = Optional.of(SWTResourceManager.getColor(SWT.COLOR_WHITE));
	private Optional<Color> enterBgColor = Optional.of(ColorConstants.SKIN_ACTIVE_BG);
	private Optional<Color> selectZtColor = Optional.of(SWTResourceManager.getColor(190, 214, 240));
	private Optional<Color> notSelectZtColor = Optional.of(SWTResourceManager.getColor(SWT.COLOR_BLACK));
	private Optional<Color> enterZtColor = Optional.of(SWTResourceManager.getColor(SWT.COLOR_BLACK));
	private boolean selection;
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public BidPriceTableItemButton(Composite parent, int style) {
		super(parent, style);
		setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		setLayout(new GridLayout(1, false));
		
		button = new Composite(this, SWT.NONE);
		button.setLayout(new GridLayout(2, false));
		GridData gd_btnNewButton = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_btnNewButton.heightHint = 44;
		gd_btnNewButton.widthHint = 160;
		button.setLayoutData(gd_btnNewButton);
		
		CLabel lblNewLabel_1 = new CLabel(button, SWT.NONE);
		lblNewLabel_1.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true, 1, 1));
		lblNewLabel_1.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/document-open.png"));
		
		textLabel = new Label(button, SWT.NONE | SWT.WRAP);
		textLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		
		init();
	}
	
	private void init() {
		bgComposite = button;
		addMouseTrackListener(button);
	}
	
	MouseTrackAdapter adapter = new MouseTrackAdapter() {
		public void mouseEnter(MouseEvent e) {
			if (!selection) enterBgColor.ifPresent(bgComposite::setBackground);
			if (!selection) enterZtColor.ifPresent(textLabel::setForeground);
		}
		
		public void mouseExit(MouseEvent e) {
			selectAction();
		}
	};
	
	public void addMouseTrackListener(Composite composite) {
		composite.addMouseTrackListener(adapter);
		Arrays.stream(composite.getChildren()).forEach(c -> {
			if (c instanceof Composite) {
				addMouseTrackListener((Composite) c);
			} else {
				c.addMouseTrackListener(adapter);
			}
		});
	}
	
	public void setExpansion(boolean expansion) {
		bgComposite = expansion ? this : button;
		if (expansion) {
			bgComposite.addMouseTrackListener(adapter);
		} else {
			bgComposite.removeMouseTrackListener(adapter);
		}
	}
	
	public void setText(String text) {
		textLabel.setText(text);
	}
	
	public String getText() {
		return textLabel.getText();
	}
	
	public void setSelection(boolean selection) {
		this.selection = selection;
		selectAction();
	}
	
	private void selectAction() {
		(selection ? selectBgColor : notSelectBgColor).ifPresent(bgComposite::setBackground);
		(selection ? selectZtColor : notSelectZtColor).ifPresent(textLabel::setForeground);
	}
	
	public void setNotSelectBgColor(Color notSelectBgColor) {
		this.notSelectBgColor = Optional.of(notSelectBgColor);
	}
	
	public void setSelectBgColor(Color selectBgColor) {
		this.selectBgColor = Optional.of(selectBgColor);
	}
	
	public void setEnterBgColor(Color enterColor) {
		this.enterBgColor = Optional.of(enterColor);
	}
	
	public void setSelectZtColor(Color selectZtColor) {
		this.selectZtColor = Optional.of(selectZtColor);
	}
	
	public void setNotSelectZtColor(Color notSelectZtColor) {
		this.notSelectZtColor = Optional.of(notSelectZtColor);
	}
	
	public void setEnterZtColor(Color enterZtColor) {
		this.enterZtColor = Optional.of(enterZtColor);
	}
	
	@Override
	public void addMouseListener(MouseListener listener) {
		addMouseListener(button, listener);
	}
	
	/**
	 * @Description:
	 * @author chenxw
	 * @param composite
	 * @param listener
	 */
	public void addMouseListener(Composite composite, MouseListener listener) {
		composite.addMouseListener(listener);
		Arrays.stream(composite.getChildren()).forEach(c -> {
			if (c instanceof Composite) {
				addMouseListener((Composite) c, listener);
			} else {
				c.addMouseListener(listener);
			}
		});
	}
	
	@Override
	public void notifyListeners(int eventType, Event event) {
		button.notifyListeners(eventType, event);
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
}
