package com.gx.obe.bpt.dialog;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.function.Supplier;

import org.eclipse.swt.core.listener.LoadingAdatper;
import org.eclipse.swt.core.widgets.ConfirmDialog;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.bpt.thread.ParseBidPriceSingleTableBiddingThread;
import com.gx.obe.common.widget.listener.ProgressMessageAdapter;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.shell.ProgressMessageDialog;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 解析招标开标一览表界面
 * @author chenxw
 */
public class ParseBiddingBidPriceDialog extends ProgressMessageDialog {
	
	private static final LoadingAdatper adatper = new LoadingAdatper() {
		public void finish(boolean flag) {
			this.setSuccess(flag);
		}
	};
	
	/**
	 * @Description: 根据项目获取消息监听
	 */
	private final Function<TenderProjectEntity, ProgressMessageListener> messageListenerFunction;
	/**
	 * @Description: 获取线程运行模块
	 */
	private final Supplier<Runnable> runnableSupplier;
	
	/**
	 * @Description: 是否解析成功
	 * @author chenxw
	 * @param parentShell
	 * @param tenderProject
	 * @param SUPPLIER_NAME
	 * @return
	 */
	public static boolean isParseSuccess(Shell parentShell, TenderProjectEntity tenderProject, String SUPPLIER_NAME) {
		if (CommonEnum.NO.equals(PropertiesUtils.getProperty(ConfigConstants.PARSE_BIDDING_BID_PRICE, CommonEnum.NO))) {
			return true;
		}
		adatper.setSuccess(false);
		new ParseBiddingBidPriceDialog(parentShell, tenderProject, adatper).open();
		if (adatper.isSuccess()) {
			return true;
		}
		ConfirmDialog confirmDialog = new ConfirmDialog(parentShell, null, "当前" + SUPPLIER_NAME + "招标开标一览表结构解析失败，是否继续解密？", null);
		initConfirmDialog(confirmDialog);
		confirmDialog.open();
		return "01".equals(confirmDialog.getClickStatus());
	}
	
	public static boolean parse(Shell parentShell, TenderProjectEntity tenderProject) {
		adatper.setSuccess(false);
		new ParseBiddingBidPriceDialog(parentShell, tenderProject, adatper).open();
		return adatper.isSuccess();
	}
	
	/**
	 * @Description: 是否解析成功
	 * @author chenxw
	 * @param parentShell
	 * @param tenderProjectList
	 * @param SUPPLIER_NAME
	 * @return
	 */
	public static boolean isParseSuccess(Shell parentShell, List<TenderProjectEntity> tenderProjectList, String SUPPLIER_NAME) {
		if (CommonEnum.NO.equals(PropertiesUtils.getProperty(ConfigConstants.PARSE_BIDDING_BID_PRICE, CommonEnum.NO))) {
			return true;
		}
		adatper.setSuccess(false);
		new ParseBiddingBidPriceDialog(parentShell, tenderProjectList, adatper).open();
		if (adatper.isSuccess()) {
			return true;
		}
		ConfirmDialog confirmDialog = new ConfirmDialog(parentShell, null, "当前项目下存在招标开标一览表结构解析失败的" + SUPPLIER_NAME + "，是否继续解密？", null);
		initConfirmDialog(confirmDialog);
		confirmDialog.open();
		return "01".equals(confirmDialog.getClickStatus());
	}
	
	/**
	 * @Description: 初始化确认界面
	 * @author chenxw
	 * @param confirmDialog
	 */
	private static void initConfirmDialog(ConfirmDialog confirmDialog) {
		confirmDialog.setTips("注：招标开标一览表结构解析失败不影响解密结果。");
		confirmDialog.setTipsColor(SWTResourceManager.getColor(255, 0, 0));
		confirmDialog.addButton(Messages.Button_continueDecrypt, "01", null);
		confirmDialog.addCancel(Messages.Button_cancel, null);
		confirmDialog.setMinimumSize(new Point(600, 250));
	}
	
	public ParseBiddingBidPriceDialog(Shell parentShell, TenderProjectEntity tenderProject, LoadingAdatper listener) {
		super(parentShell, true, -1, listener);
		this.setTitle("正在解析招标开标一览表结构");
		
		this.messageListenerFunction = t -> this;
		this.runnableSupplier = () -> createParseBiddingBidPriceThread(tenderProject);
	}
	
	public ParseBiddingBidPriceDialog(Shell parentShell, List<TenderProjectEntity> tenderProjectList, LoadingAdatper listener) {
		super(parentShell, true, -1, listener);
		this.setTitle("正在解析招标开标一览表结构");
		
		List<TenderProjectEntity> failList = Collections.synchronizedList(new ArrayList<>());
		
		this.messageListenerFunction = t -> new parseMultiplePackMessageAdapter(t, failList);
		this.runnableSupplier = () -> () -> parseMultiplePack(tenderProjectList, failList);
	}
	
	/**
	 * @Description: 解析多个标段的消息适配器
	 * @author chenxw
	 */
	class parseMultiplePackMessageAdapter extends ProgressMessageAdapter {
		
		private TenderProjectEntity tenderProject;
		private List<TenderProjectEntity> failList;
		
		public parseMultiplePackMessageAdapter(TenderProjectEntity tenderProject, List<TenderProjectEntity> failList) {
			this.tenderProject = tenderProject;
			this.failList = failList;
		}
		
		@Override
		public void finish(boolean flag) {
			if (flag) {
				ParseBiddingBidPriceDialog.super.message("[" + tenderProject.getTenderNo() + "]" + tenderProject.getTenderName() + "-解析招标开标一览表成功", 1);
			} else {
				ParseBiddingBidPriceDialog.super.message("[" + tenderProject.getTenderNo() + "]" + tenderProject.getTenderName() + "-解析招标开标一览表失败", -1);
				failList.add(tenderProject);
			}
		}
		
	}
	
	/**
	 * @Description: 解析多个包件
	 * @author chenxw
	 * @param tenderProjectList
	 * @param failList
	 */
	private void parseMultiplePack(List<TenderProjectEntity> tenderProjectList, List<TenderProjectEntity> failList) {
		int size = tenderProjectList.size();
		ExecutorService cachedThreadPool = Executors.newScheduledThreadPool(size > 4 ? 4 : size);
		tenderProjectList.stream().map(this::createParseBiddingBidPriceThread).forEach(cachedThreadPool::submit);
		cachedThreadPool.shutdown();
		try {
			if (!cachedThreadPool.awaitTermination(1, TimeUnit.HOURS)) {
				fail("连接超时");
				return;
			}
		} catch (InterruptedException e) {
			fail("异常中断");
			return;
		}
		if (!failList.isEmpty()) {
			fail("存在解析失败的标段");
			return;
		}
		message("解析成功", 1);
		finish(true);
	}
	
	/**
	 * @Description: 创建解析开标一览表线程
	 * @author chenxw
	 * @param tenderProject
	 * @return
	 */
	private Runnable createParseBiddingBidPriceThread(TenderProjectEntity tenderProject) {
		ParseBidPriceSingleTableBiddingThread biddingThread = new ParseBidPriceSingleTableBiddingThread(tenderProject);
		biddingThread.addProgressMessageListener(messageListenerFunction.apply(tenderProject));
		return biddingThread;
	}
	
	/**
	 * @Description: 失败
	 * @author chenxw
	 * @param message
	 */
	public void fail(String message) {
		message(message, -1);
		finish(false);
	}
	
	@Override
	public void open() {
		new Thread(runnableSupplier.get(), "解析招标开标一览表线程").start();
		super.open();
	}
	
}
