package com.gx.obe.bpt.parse;

import java.util.List;
import java.util.Optional;

import com.gx.obe.bpt.entity.BidPriceTable;
import com.gx.obe.bpt.entity.BidPriceTableItem;
import com.gx.obe.bpt.entity.BidPriceTableTitle;
import com.gx.obe.bpt.entity.BidPriceTableValue;
import com.gx.obe.bpt.enumeration.BidPriceBusinessType;
import com.gx.obe.bpt.enumeration.BidPriceTableType;
import com.gx.obe.bpt.service.BidPriceTableService;
import com.gx.obe.bpt.utils.ParseBidPriceTableUtils;
import com.gx.obe.bpt.utils.ParseBidPriceTableUtils.ItemStreamListener;
import com.gx.obe.bpt.vo.BidPriceTableDateInfo;
import com.gx.obe.util.utils.UuidUtils;

public class ParseBidPriceSingleTable {
	
	private final String BIDDING_ID = "bidding_id";
	private final BidPriceTableService bidPriceTableService = new BidPriceTableService();
	private final BidPriceTableDateInfo bidPriceTableDateInfo = new BidPriceTableDateInfo();
	
	private String tenderId;
	private String supplierId;
	private String businessType;
	
	/**
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 */
	public ParseBidPriceSingleTable(String tenderId, String supplierId) {
		this.tenderId = tenderId;
		this.supplierId = supplierId;
		this.businessType = BidPriceBusinessType.SUPPLIER.getKey();
	}
	
	/**
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 */
	public ParseBidPriceSingleTable(String tenderId) {
		this.tenderId = tenderId;
		this.supplierId = BIDDING_ID;
		this.businessType = BidPriceBusinessType.BIDDING.getKey();
	}
	
	/**
	 * @Description: 解析
	 * @author chenxw
	 * @param detailTrElementList
	 * @return 
	 */
	public boolean parse(List<?> detailTrElementList) {
		return Optional.ofNullable(detailTrElementList).map(l -> {
			resolverDetail(l);
			return bidPriceTableService.deleteOrSaveBidPriceTableDateInfo(tenderId, supplierId, bidPriceTableDateInfo);
		}).orElse(true);
	}
	
	/**
	 * @Description: 解析明细
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private void resolverDetail(List<?> detailTrElementList) {
		BidPriceTable bidPriceDetailTable = initBidPriceTable();
		
		bidPriceTableDateInfo.getBidPriceTableList().add(bidPriceDetailTable);
		
		//解析报价明细表头
		List<BidPriceTableTitle> bidPriceTableTitleList = ParseBidPriceTableUtils.resolverBidPriceTableTitleStream(
				detailTrElementList, 
				tenderId, 
				supplierId, 
				bidPriceDetailTable.getId());
		
		bidPriceTableDateInfo.getBidPriceTableTitleList().addAll(bidPriceTableTitleList);
		
		ItemStreamListener itemStreamListener = (tableItemId, trElement) -> {
			//解析报价明细内容
			List<BidPriceTableValue> bidPriceTableValueList = ParseBidPriceTableUtils.resolverBidPriceTableValueStream(
					trElement.elements(), 
					tenderId, 
					supplierId, 
					bidPriceDetailTable.getId(), 
					tableItemId, 
					i -> bidPriceTableTitleList.stream()
						.filter(t -> t.getSortNo() - 1 == i)
						.findAny()
						.map(BidPriceTableTitle::getId)
						.orElseThrow(RuntimeException::new));
			
			bidPriceTableDateInfo.getBidPriceTableValueList().addAll(bidPriceTableValueList);
		};
		
		//解析报价明细项
		List<BidPriceTableItem> bidPriceTableItemList = ParseBidPriceTableUtils.resolverBidPriceTableItemStream(
				detailTrElementList, 
				tenderId, 
				supplierId, 
				bidPriceDetailTable.getId(), 
				null, 
				null, 
				itemStreamListener);
		
		bidPriceTableDateInfo.getBidPriceTableItemList().addAll(bidPriceTableItemList);
	}
	
	/**
	 * @Description: 初始化PriceTable
	 * @author chenxw
	 * @param element
	 * @param bidPriceTableType
	 * @return
	 */
	private BidPriceTable initBidPriceTable() {
		BidPriceTable bidPriceTable = new BidPriceTable();
		bidPriceTable.setId(UuidUtils.getUUID());
		bidPriceTable.setTenderId(tenderId);
		bidPriceTable.setSupplierId(supplierId);
		bidPriceTable.setTableType(BidPriceTableType.DETAIL.getKey());
		bidPriceTable.setBusinessType(businessType);;
		return bidPriceTable;
	}
	
}
