package com.gx.obe.bpt.parse;

import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import org.dom4j.Element;

import com.gx.obe.bpt.entity.BidPriceTable;
import com.gx.obe.bpt.entity.BidPriceTableItem;
import com.gx.obe.bpt.entity.BidPriceTableItemCollect;
import com.gx.obe.bpt.entity.BidPriceTableTitle;
import com.gx.obe.bpt.entity.BidPriceTableValue;
import com.gx.obe.bpt.enumeration.BidPriceBusinessType;
import com.gx.obe.bpt.enumeration.BidPriceTableType;
import com.gx.obe.bpt.service.BidPriceTableService;
import com.gx.obe.bpt.utils.ParseBidPriceTableUtils;
import com.gx.obe.bpt.utils.ParseBidPriceTableUtils.ItemStreamListener;
import com.gx.obe.bpt.vo.BidPriceTableDateInfo;
import com.gx.obe.util.utils.UuidUtils;

public class ParseBidPriceTable {
	
	private final BidPriceTableService bidPriceTableService = new BidPriceTableService();
	
	private final BidPriceTableDateInfo bidPriceTableDateInfo = new BidPriceTableDateInfo();
	
	private String tenderId;
	private String supplierId;
	
	/**
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 */
	public ParseBidPriceTable(String tenderId, String supplierId) {
		this.tenderId = tenderId;
		this.supplierId = supplierId;
	}
	
	public boolean parse(Supplier<Element> supplier, Function<String, List<?>> function) {
		return Optional.ofNullable(supplier.get()).map(e -> {
			resolverDetail(e, function);
			resolverCollect(e, function);
			return bidPriceTableService.deleteOrSaveBidPriceTableDateInfo(tenderId, supplierId, bidPriceTableDateInfo);
		}).orElse(false);
	}
	
	/**
	 * @Description: 获取报价明细ID
	 * @author chenxw
	 * @param bidPriceTableElementList
	 * @param function
	 * @return
	 */
	private Function<String, String> getbidPriceTableIdFunction(List<?> bidPriceTableElementList, Function<Element, BidPriceTable> function) {
		Optional<List<BidPriceTable>> bidPriceTableList = Optional.ofNullable(bidPriceTableElementList)
				.map(List::stream)
				.map(s -> s
						.map(t -> (Element) t)
						.map(function)
						.peek(t -> t.setBusinessType(BidPriceBusinessType.BID_PRICE_TABLE_ITEM.getKey()))
						.collect(Collectors.toList()));
		return id -> bidPriceTableList
				.map(List::stream)
				.flatMap(s -> s
						.filter(t -> id.equals(t.getSubjectTableId()))
						.findAny())
				.map(BidPriceTable::getId).orElse(null);
	}
	
	/**
	 * @Description: 解析明细
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private BidPriceTable resolverDetail(Element element, Function<String, List<?>> function) {
		List<?> detailTrElementList = Optional.ofNullable(element)
				.map(e -> e.elementTextTrim("detailFileName"))
				.map(function)
				.orElseThrow(() -> new RuntimeException("detailTrElementList is null : " + element.elementTextTrim("totalFileName")));
		
		BidPriceTable bidPriceDetailTable = initBidPriceTable(element, BidPriceTableType.DETAIL);
		
		bidPriceTableDateInfo.getBidPriceTableList().add(bidPriceDetailTable);
		
		List<?> bidPriceTableElementList = ParseBidPriceTableUtils.resolverBidPriceTableElementStream(element);
		
		Function<String, String> subDetalTableIdFunction = getbidPriceTableIdFunction(bidPriceTableElementList, e -> resolverDetail(e, function));
		Function<String, String> subCollectTableIdFunction = getbidPriceTableIdFunction(bidPriceTableElementList, e -> resolverCollect(e, function));
		
		//解析报价明细表头
		List<BidPriceTableTitle> bidPriceTableTitleList = ParseBidPriceTableUtils.resolverBidPriceTableTitleStream(
				detailTrElementList, 
				tenderId, 
				supplierId, 
				bidPriceDetailTable.getId());
		
		bidPriceTableDateInfo.getBidPriceTableTitleList().addAll(bidPriceTableTitleList);
		
		ItemStreamListener itemStreamListener = (tableItemId, trElement) -> {
			//解析报价明细内容
			List<BidPriceTableValue> bidPriceTableValueList = ParseBidPriceTableUtils.resolverBidPriceTableValueStream(
					trElement.elements(), 
					tenderId, 
					supplierId, 
					bidPriceDetailTable.getId(), 
					tableItemId, 
					i -> bidPriceTableTitleList.stream()
						.filter(t -> t.getSortNo() - 1 == i)
						.findAny()
						.map(BidPriceTableTitle::getId)
						.orElseThrow(RuntimeException::new));
			
			bidPriceTableDateInfo.getBidPriceTableValueList().addAll(bidPriceTableValueList);
		};
		
		//解析报价明细项
		List<BidPriceTableItem> bidPriceTableItemList = ParseBidPriceTableUtils.resolverBidPriceTableItemStream(
				detailTrElementList, 
				tenderId, 
				supplierId, 
				bidPriceDetailTable.getId(), 
				subDetalTableIdFunction, 
				subCollectTableIdFunction, 
				itemStreamListener);
		
		bidPriceTableDateInfo.getBidPriceTableItemList().addAll(bidPriceTableItemList);
		
		return bidPriceDetailTable;
	}
	
	/**
	 * @Description: 解析汇总
	 * @author chenxw
	 * @param element
	 * @param aaa 
	 * @return
	 */
	private BidPriceTable resolverCollect(Element element, Function<String, List<?>> function) {
		List<?> totalTrElementList = Optional.ofNullable(element)
				.map(e -> e.elementTextTrim("totalFileName"))
				.map(function)
				.orElseThrow(() -> new RuntimeException("totalTrElementList is null : " + element.elementTextTrim("totalFileName")));
		
		BidPriceTable bidPriceTotalTable = initBidPriceTable(element, BidPriceTableType.COLLECT);
		
		bidPriceTableDateInfo.getBidPriceTableList().add(bidPriceTotalTable);
		
		//解析报价明细汇总内容
		List<BidPriceTableItemCollect> bidPriceTableItemCollectList = ParseBidPriceTableUtils.resolverBidPriceTableItemCollectStream(
				totalTrElementList, 
				tenderId, 
				supplierId, 
				bidPriceTotalTable.getId());
		
		bidPriceTableDateInfo.getBidPriceTableItemCollectList().addAll(bidPriceTableItemCollectList);
		
		return bidPriceTotalTable;
	}
	
	/**
	 * @Description: 初始化PriceTable
	 * @author chenxw
	 * @param element
	 * @param bidPriceTableType
	 * @return
	 */
	private BidPriceTable initBidPriceTable(Element element, BidPriceTableType bidPriceTableType) {
		if (element == null || bidPriceTableType == null) {
			throw new RuntimeException();
		}
		BidPriceTable bidPriceTable = new BidPriceTable();
		bidPriceTable.setId(UuidUtils.getUUID());
		bidPriceTable.setTenderId(tenderId);
		bidPriceTable.setSupplierId(supplierId);
		bidPriceTable.setTableType(bidPriceTableType.getKey());
		bidPriceTable.setBusinessType(BidPriceBusinessType.SUPPLIER.getKey());
		bidPriceTable.setSubjectTableId(element.elementTextTrim("subjectTableID"));
		return bidPriceTable;
	}
	
}
