package com.gx.obe.bpt.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.bpt.entity.BidPriceTable;
import com.gx.obe.bpt.vo.BidPriceTableDateInfo;
import com.gx.obe.web.entity.Result.RequestBody;
import com.gx.obe.web.utils.ObeHttpUtils;

/**
 * @Description:
 * @author chenxw
 */
public class BidPriceTableService {
	
	protected static final Logger logger = Logger.getLogger(BidPriceTableService.class);
	
	protected static final String URL = "obeBidPriceTable";
	
	/**
	 * @Description: 清空并保存投标报价明细表信息数据
	 * @author chenxw
	 * @param tenderId
	 * @param bidPriceTableDateInfo
	 * @return
	 */
	public boolean deleteOrSaveBidPriceTableDateInfo(String tenderId, String supplierId, BidPriceTableDateInfo bidPriceTableDateInfo) {
		RequestBody<BidPriceTableDateInfo> requestBody = new RequestBody<>(bidPriceTableDateInfo);
		requestBody.setParam("tenderId", tenderId);
		requestBody.setParam("supplierId", supplierId);
		return ObeHttpUtils.post(URL.concat("/deleteOrSaveBidPriceTableDateInfo"), requestBody);
	}
	
	/**
	 * @Description: 根据供应商id获取投标报价表
	 * @author chenxw
	 * @param supplierId
	 * @return
	 */
	public List<BidPriceTable> getBidPriceTableInfoBySupplierId(String tenderId, String supplierId) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		param.put("supplierId", supplierId);
		return ObeHttpUtils.getList(URL.concat("/getBidPriceTableInfoBySupplierId"), param, BidPriceTable.class);
	}
	
	/**
	 * @Description: 根据供应商id获取投标报价明细表
	 * @author chenxw
	 * @param supplierId
	 * @return
	 */
	public BidPriceTable getBidPriceTableDetailBySupplierId(String tenderId, String supplierId) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		param.put("supplierId", supplierId);
		return ObeHttpUtils.getOne(URL.concat("/getBidPriceTableDetailBySupplierId"), param, BidPriceTable.class);
	}
	
	/**
	 * @Description: 根据id获取投标报价表
	 * @author chenxw
	 * @param id
	 * @return
	 */
	public BidPriceTable getBidPriceTableInfoById(String id) {
		Map<String, Object> param = new HashMap<>();
		param.put("id", id);
		return ObeHttpUtils.getOne(URL.concat("/getBidPriceTableInfoById"), param, BidPriceTable.class);
	}
	
	/**
	 * @Description: 根据id获取投标报价表
	 * @author chenxw
	 * @param id
	 * @return
	 */
	public BidPriceTable getBiddingTable(String tenderId) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		return ObeHttpUtils.getOne(URL.concat("/getBiddingTable"), param, BidPriceTable.class);
	}
	
}
