package com.gx.obe.bpt.thread;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.bpt.parse.ParseBidPriceSingleTable;
import com.gx.obe.bpt.utils.BidPriceUtils;
import com.gx.obe.bpt.utils.ParseBidPriceTableUtils;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.BidPriceService;
import com.gx.obe.web.service.ProjectRuleService;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 解析招标文件中报价明细表线程
 * @author chenxw
 */
public class ParseBidPriceSingleTableBiddingThread implements Runnable {
	
	private static final Logger LOG = Logger.getLogger(ParseBidPriceSingleTableBiddingThread.class);
	
	private final BidPriceService bidPriceService = new BidPriceService();
	private final ProjectRuleService projectRuleService = new ProjectRuleService();
	
	private final TenderProjectEntity tenderProject;
	private final HttpUrl httpUrl;
	
	private ProgressMessageListener listener;
	
	private boolean flag = true;

	
	/**
	 * @author chenxw
	 * @param tenderProject
	 */
	public ParseBidPriceSingleTableBiddingThread(TenderProjectEntity tenderProject) {
		this.tenderProject = tenderProject;
		this.httpUrl = HttpUtils.getProjectHttpUrl(tenderProject, "bidOpeningHS.do?getBiddingPrice");
	}
	
	@Override
	public void run() {
		this.listener = Optional.ofNullable(listener).orElse(ProgressMessageListener.EMPTY);
		
		if (Objects.isNull(tenderProject)) {
			fail("项目信息为空！");
			return;
		}
		// 获取平台返回结果
		println("开始获取招标开标一览表结构！");
		HttpResponse httpResponse = HttpUtils.doPost(httpUrl);
		if (!httpResponse.getStatus()) {
			fail("失败原因：".concat(httpResponse.getMsg()));
			finish();
			return;
		}
		// 获取返回结果中data数据
		JSONArray jsonArray = httpResponse.getDataJSONArray("bidPriceFileList");
		if (Objects.isNull(jsonArray)) {
			fail("未获取到招标开标一览表结构！");
			finish();
			return;
		}
		Object[] array = jsonArray.toArray();
		if (array.length == 0) {
			fail("未获取到招标开标一览表结构！");
			finish();
			return;
		}
		Arrays.stream(array).map(this::toJSONObject).collect(Collectors.toMap(getFun("fileName"), getFun("fileContent"))).forEach((key, value) -> {
			if (key.startsWith("MulPriceGrid_")) {
				println("开始解析报价明细表结构！");
				parsePriceDetail(value);
			}
			if (key.startsWith("BidPrice_")) {
				println("开始解析开标一览表结构！");
				parseBidPrice(value);
			}
			if (key.startsWith("BidPriceFileItem_")) {
				println("开始解析投标报价文件项！");
				parseBidPriceFileItem(value);
			}
			if (key.startsWith("MulPriceFileItem_")) {
				println("开始解析报价明细文件项");
				parseMulPriceFileItem(value);
			}
		});
		finish();
	}
	
	private void parseMulPriceFileItem(String value) {
		Document doc = Optional.ofNullable(value).map(ParseBidPriceTableUtils::DocumentHelperParseText).orElse(null);
		if (Objects.isNull(doc)) {
			fail("解析失败！");
			return;
		}
		parseBidPriceInfo(doc.getRootElement());
	}
	
	private void parseBidPriceFileItem(String value) {
		Document doc = Optional.ofNullable(value).map(ParseBidPriceTableUtils::DocumentHelperParseText).orElse(null);
		if (Objects.isNull(doc)) {
			fail("解析失败！");
			return;
		}
		parseBidPriceInfo(doc.getRootElement().element("rightHeader"));
	}
	
	private void parseBidPriceInfo(Element element) {
		TenderProjectRuleEntity projectRule = projectRuleService.getProjectRule(tenderProject.getId());
		projectRule.setPriceUnit(element.elementTextTrim("unit"));
		projectRule.setCurrencyCode(element.elementTextTrim("currencyCode"));
		projectRule.setCurrencyName(element.elementTextTrim("currencyName"));
		if (!projectRuleService.updateAssignProperty(projectRule, new String[] { "priceUnit", "currencyCode", "currencyName" })) {
			fail("保存投标报价信息失败！");
			return;
		}
		success("解析成功！");
	}
	
	/**
	 * @Description: 解析报价明细
	 * @author chenxw
	 * @param value
	 */
	private void parsePriceDetail(String value) {
		// 获取data数据中明细表tr列表
		List<?> detailTrElementList = Optional.ofNullable(value).map(ParseBidPriceTableUtils::DocumentHelperParseText).map(ParseBidPriceTableUtils::resolverElementList).orElse(null);
		if (CollectionUtils.isNull(detailTrElementList)) {
			fail("未解析到报价明细表结构！");
			return;
		}
		// 创建报价明细xml解析器
		if (!new ParseBidPriceSingleTable(tenderProject.getId()).parse(detailTrElementList)) {
			fail("解析失败！");
			return;
		}
		success("解析成功！");
	}
	
	/**
	 * @Description: 解析开标一览表结构
	 * @author chenxw
	 * @param value
	 */
	private void parseBidPrice(String value) {
		if (CollectionUtils.isNotNull(bidPriceService.getTenderBidPriceTableList(tenderProject.getId()))) {
			success("开标一览表结构已存在！");
			return;
		}
		Document doc = Optional.ofNullable(value).map(ParseBidPriceTableUtils::DocumentHelperParseText).orElse(null);
		if (Objects.isNull(doc)) {
			fail("解析失败！");
			return;
		}
		List<BidPrice> bidPriceList = BidPriceUtils.getBidPriceTable(BidPriceUtils.getSupplierBidPriceResultTable(tenderProject.getId(), null, doc));
		if (CollectionUtils.isNull(bidPriceList)) {
			fail("未解析到开标一览表结构！");
			return;
		}
		if (!bidPriceService.batchSaveResolverBidPriceTable(bidPriceList)) {
			fail("保存开标一览表结构失败！");
			return;
		}
		success("解析成功！");
	}
	
	/**
	 * @Description: 失败
	 * @author chenxw
	 * @param message
	 */
	private void success(String message) {
		listener.message(message, 1);
	}
	
	/**
	 * @Description: 失败
	 * @author chenxw
	 * @param message
	 */
	private void fail(String message) {
		listener.message(message, -1);
		flag = false;
		LOG.error(Optional.ofNullable(httpUrl.getName()).orElse("接口请求") + (flag ? "成功" : "失败") + "\r\n"
				+ "接口：" + httpUrl.getUrl() + "\t\n"
				+ "接口参数：" + httpUrl.getAttributs() + "\t\n"
				+ "接口返回内容：" + Optional.ofNullable(httpUrl.getHttpResponse()).map(HttpResponse::getContent).orElse("null"));
	}
	
	private void finish() {
		println("解析完成！");
		listener.finish(flag);
	}
	
	/**
	 * @Description: 打印
	 * @author chenxw
	 * @param message
	 */
	private void println(String message) {
		listener.message(message, 0);
	}
	
	private JSONObject toJSONObject(Object t) {
		if (t instanceof JSONObject) {
			return (JSONObject) t;
		}
		return null;
	}
	
	private Function<JSONObject, String> getFun(String key) {
		return t -> t.getString(key);
	}
	
	public void addProgressMessageListener(ProgressMessageListener listener) {
		this.listener = listener;
	}
	
}
