package com.gx.obe.bpt.utils;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.common.file.entity.BiddingFile;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.components.core.enumeration.BidPriceEnum;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.utils.NowTimeUtils;

public class BidPriceUtils {
	
	private static Logger LOG = Logger.getLogger(BidPriceUtils.class);
	
	/**
	 * @Description: 解析供应商开标一览表
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static List<BidPriceResult> getSupplierBidPriceResultTable(String tenderId, String supplierId) {
		
		// 获得开标一览表的投标报价
		File xmlFile = new File(FileConstants.getBidFilePath(tenderId, supplierId) + FileConstants.BID_PRICE_XML);
		if (null != xmlFile && xmlFile.exists()) {
			Document doc = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
			if (null != doc) {
				return getSupplierBidPriceResultTable(tenderId, supplierId, doc);
			} else {
				LOG.error("tenderId:" + tenderId + "\tsupplierId:" + supplierId + "\t报价文件读取错误" + xmlFile.getAbsolutePath());
			}
		} else {
			LOG.error("tenderId:" + tenderId + "\tsupplierId:" + supplierId + "\t报价文件不存在");
		}
		return null;
	}
	
	/**
	 * @Description: 获取项目的开标一览表结构
	 * @author guoyr
	 * @param tenderId
	 * @param tenderNo
	 * @return
	 */
	public static List<BidPrice> getBidPriceTable(String tenderId, String tenderNo) {
		String curBidPriceId = null;
		// 获得开标一览表的投标报价索引文件
		File indexFile = new File(FileConstants.getBiddingFilePath(tenderId) + BiddingFile.BIDPRICE_TABLE_INDEX);
		if (null != indexFile && indexFile.exists()) {
			Document indexDoc = XmlUtils.getXmlDocument(indexFile.getAbsolutePath());
			if (null != indexDoc) {
				Element rootElement = indexDoc.getRootElement();// 声明跟元素
				List<?> bidPriceTableItemInfoLinst = rootElement.elements("bidPriceTableItemInfo");
				if (null != bidPriceTableItemInfoLinst && bidPriceTableItemInfoLinst.size() > 0) {
					for (Object bidPriceTableItemInfo : bidPriceTableItemInfoLinst) {
						Element bidPriceItemInfo = (Element) bidPriceTableItemInfo;
						String curpackCode = bidPriceItemInfo.elementTextTrim("packCode");
						if (null != curpackCode && curpackCode.equals(tenderNo)) {
							curBidPriceId = bidPriceItemInfo.elementTextTrim("packId");
							break;
						}
					}
				}
			}
		}
		
		File xmlFile = null;
		if (StringUtils.hasLength(curBidPriceId)) {
			xmlFile = new File(FileConstants.getBiddingFilePath(tenderId) + "BidPrice_" + curBidPriceId + ".xml");
		} else {
			xmlFile = new File(FileConstants.getBiddingFilePath(tenderId) + FileConstants.BID_PRICE_XML);
		}
		return getBidPriceTableFromFile(tenderId, xmlFile.getAbsolutePath());
	}
	
	/**
	 * @Description: 读取开标一览表模板
	 * @author guoyr
	 * @param tenderId
	 * @param xmlPath
	 * @return
	 */
	public static List<BidPrice> getBidPriceTableFromFile(String tenderId, String xmlPath) {
		
		List<BidPrice> bidPriceList = null;
		
		File xmlFile = new File(xmlPath);
		Document doc = null;
		if (null != xmlFile && xmlFile.exists()) {
			doc = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
			if (null != doc) {
				bidPriceList = getBidPriceTable(getSupplierBidPriceResultTable(tenderId, null, doc));
			} else {
				LOG.error("tenderId:" + tenderId + "\t开标一览表结构文件读取错误" + xmlFile.getAbsolutePath());
			}
		} else {
			LOG.error("tenderId:" + tenderId + "\t开标一览表结构文件不存在");
		}
		
		return bidPriceList;
		
	}
	
	/**
	 * @Description: 获取开标一览表结构
	 * @author guoyr
	 * @param bidPriceResultList
	 * @return
	 */
	public static List<BidPrice> getBidPriceTable(List<BidPriceResult> bidPriceResultList) {
		List<BidPrice> bidPriceList = null;
		if (null != bidPriceResultList && bidPriceResultList.size() > 0) {
			bidPriceList = new ArrayList<BidPrice>();
			int sortNo = 1;
			for (BidPriceResult bidPriceResult : bidPriceResultList) {
				BidPrice bidPrice = new BidPrice();
				bidPrice.setId(UuidUtils.getUUID());
				bidPrice.setTenderId(bidPriceResult.getTenderId());
				bidPrice.setBidPriceName(bidPriceResult.getBidPriceName());
				if (BidPriceEnum.IS_PRICE_ITEM.equals(bidPriceResult.getBidPriceType())) {
					bidPrice.setBidPriceType(bidPriceResult.getBidPriceType());
					bidPrice.setJoinTotal(BidPriceEnum.JOIN_TOTAL.equals(bidPriceResult.getJoinTotal()) ? BidPriceEnum.JOIN_TOTAL : BidPriceEnum.NOT_JOIN_TOTAL);
					bidPrice.setPriceType(BidPriceEnum.IS_RATE.equals(bidPriceResult.getPriceType()) ? BidPriceEnum.IS_RATE : BidPriceEnum.IS_PRICE);
				} else {
					bidPrice.setBidPriceType(BidPriceEnum.NOT_IS_PRICE_ITEM);
					bidPrice.setJoinTotal(BidPriceEnum.NOT_JOIN_TOTAL);
				}
				bidPrice.setPriceUnit(bidPriceResult.getPriceUnit());
				if (null != bidPriceResult.getBidPriceCode() && bidPriceResult.getBidPriceCode().length() > 0) {
					bidPrice.setBidPriceCode(bidPriceResult.getBidPriceCode());
				} else {
					bidPrice.setBidPriceCode(bidPrice.getId());
				}
				bidPrice.setSortNo(sortNo++);
				bidPriceList.add(bidPrice);
			}
		}
		return bidPriceList;
	}
	
	/**
	 * @Description: 获得供应商投标报价结果
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static List<BidPriceResult> getSupplierBidPriceResultTable(String tenderId, String supplierId, String xmlPath) {
		File xmlFile = new File(xmlPath);
		if (null != xmlFile && xmlFile.exists()) {
			Document doc = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
			if (null != doc) {
				return getSupplierBidPriceResultTable(tenderId, supplierId, doc);
			} else {
				LOG.error("tenderId:" + tenderId + "\tsupplierId:" + supplierId + "\t报价文件读取错误" + xmlFile.getAbsolutePath());
			}
		} else {
			LOG.error("tenderId:" + tenderId + "\tsupplierId:" + supplierId + "\t报价文件不存在");
		}
		return null;
	}
	
	/**
	 * @Description: 获得供应商投标报价结果
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static List<BidPriceResult> getSupplierBidPriceResultTable(String tenderId, String supplierId, Document doc) {
		List<BidPriceResult> bidPriceResultList = null;
		if (null != doc) {
			bidPriceResultList = new ArrayList<BidPriceResult>();
			Element rootElement = doc.getRootElement();
			// 新定义报价格式
			Element bidPriceElement = rootElement;
			if (null != rootElement.elements("bidPrice")) {
				// 兼容以前多报价树形结构格式
				bidPriceElement = (Element) rootElement.elements("bidPrice").get(0);
			}
			List<?> bidPriceListElements = bidPriceElement.elements();
			if (null != bidPriceListElements && bidPriceListElements.size() > 0) {
				int sortNo = 1;
				Date createTime = NowTimeUtils.getNowDate();
				for (Object itemObj : bidPriceListElements) {
					BidPriceResult bidPriceResult = null;
					Element itemElement = (Element) itemObj;
					boolean isPriceItem = false;// 是否报价项
					if ("priceItem".equals(itemElement.getName())) {
						// 兼容以前报价项
						isPriceItem = true;
					} else {
						
						// 新定义报价项属性isPriceItem
						Element isPriceItemElement = itemElement.element("isPriceItem");
						if (null != isPriceItemElement) {
							isPriceItem = BidPriceEnum.IS_PRICE_ITEM.equals(isPriceItemElement.getTextTrim());
						}
					}
					if (isPriceItem) {
						bidPriceResult = getBidPriceResult(itemElement);
						if (null == bidPriceResult.getBidPriceName()) {
							continue;
						}
					} else {
						bidPriceResult = getBidPriceItem(itemElement);
					}
					bidPriceResult.setCreateTime(createTime);
					bidPriceResult.setTenderId(tenderId);
					bidPriceResult.setSupplierId(supplierId);
					bidPriceResultList.add(bidPriceResult);
					bidPriceResult.setSortNo(sortNo++);
				}
			}
		}
		return bidPriceResultList;
		
	}
	
	/**
	 * @Description: 解析报价项
	 * @author guoyr
	 * @param priceItemElement
	 * @return
	 */
	private static BidPriceResult getBidPriceResult(Element priceItemElement) {
		BigDecimal bidPrice = null;
		boolean isRate = BidPriceEnum.IS_RATE.equals(priceItemElement.elementTextTrim("priceType"));//<!--报价类型 00 金额 01 费率-->
		boolean joinTotal = BidPriceEnum.JOIN_TOTAL.equals(priceItemElement.elementTextTrim("joinTotal"));//<!--是否计入总价 01 计入 00不计入-->
		BidPriceResult bidPriceResult = new BidPriceResult();
		bidPriceResult.setBidPriceName(ObjectUtils.getObjString(priceItemElement.elementTextTrim("name")));
		bidPriceResult.setCurrencyCode(priceItemElement.elementText("currencyCode"));
		bidPriceResult.setCurrencyName(priceItemElement.elementTextTrim("currencyName"));
		bidPriceResult.setPriceUnit(priceItemElement.elementTextTrim("unit"));
		if (bidPriceResult.getPriceUnit().isEmpty()) {
			bidPriceResult.setPriceUnit(CommonEnum.getPriceUintCN(CommonEnum.YUAN));
		}
		String itemId = priceItemElement.elementTextTrim("id");
		
		// @Deprecated 新版本将删除该项
		bidPriceResult.setPriceContent(priceItemElement.elementTextTrim("contentValue"));
		
		bidPrice = ObjectUtils.getObjBigDecimal(priceItemElement.elementText("value"));
		if (null != bidPrice) {
			if (bidPrice.compareTo(new BigDecimal("9999999999999999")) > 0) {
				bidPrice = new BigDecimal("-1");
				LOG.error(bidPrice.toString() + "投标报价超出范围" + priceItemElement.asXML());
			}
		}
		
		bidPriceResult.setBidPrice(bidPrice);
		bidPriceResult.setFinalBidPrice(bidPrice);
		bidPriceResult.setEvaluationPrice(bidPrice);
		bidPriceResult.setBidPriceId(itemId);
		String itemCode = priceItemElement.elementTextTrim("code");
		if (StringUtils.isEmpty(itemCode)) {
			itemCode = UuidUtils.getUUID();
		}
		bidPriceResult.setBidPriceCode(itemCode);
		bidPriceResult.setBidPriceType(BidPriceEnum.IS_PRICE_ITEM);
		bidPriceResult.setJoinTotal(joinTotal ? BidPriceEnum.JOIN_TOTAL : BidPriceEnum.NOT_JOIN_TOTAL);
		bidPriceResult.setPriceType(isRate ? BidPriceEnum.IS_RATE : null);
		return bidPriceResult;
	}
	
	/**
	 * @Description: 解析非报价项
	 * @author guoyr
	 * @param priceItemElement
	 * @return
	 */
	private static BidPriceResult getBidPriceItem(Element priceItemElement) {
		BidPriceResult bidPriceResult = new BidPriceResult();
		bidPriceResult.setBidPriceName(ObjectUtils.getObjString(priceItemElement.elementTextTrim("name")));
		String itemContent = priceItemElement.elementTextTrim("value");
		String itemId = priceItemElement.elementTextTrim("id");
		if (null != itemContent && itemContent.length() > 0) {
			BigDecimal bidPrice = ObjectUtils.getObjBigDecimal(itemContent);
			if (null != bidPrice) {
				if (bidPrice.compareTo(new BigDecimal("9999999999999999")) > 0) {
					bidPrice = new BigDecimal("-1");
					LOG.error(bidPrice.toString() + "投标报价超出范围" + priceItemElement.asXML());
				}
				bidPriceResult.setBidPrice(bidPrice);
				// @Deprecated 新版本将删除该项
				String priceContent = priceItemElement.elementTextTrim("contentValue");
				if (null != priceContent && priceContent.length() > 0) {
					bidPriceResult.setPriceContent(priceContent);
				} else {
					bidPriceResult.setPriceContent(itemContent);
				}
			} else {
				bidPriceResult.setPriceContent(itemContent);
			}
		}
		bidPriceResult.setBidPriceId(itemId);
		String itemCode = priceItemElement.elementTextTrim("code");
		if (StringUtils.isEmpty(itemCode)) {
			itemCode = UuidUtils.getUUID();
		}
		bidPriceResult.setBidPriceCode(itemCode);
		String itemUnit = priceItemElement.elementTextTrim("unit");
		if (null != itemUnit && itemUnit.length() > 0) {
			// 招标文件中会出现非报价项为元的情况
//			bidPriceResult.setPriceUnit(itemUnit);
		}
		bidPriceResult.setBidPriceType(BidPriceEnum.NOT_IS_PRICE_ITEM);
		bidPriceResult.setJoinTotal(BidPriceEnum.NOT_JOIN_TOTAL);
		bidPriceResult.setBidPriceId(itemId);
		return bidPriceResult;
	}
	
	/**
	 * 
	 */
	public static BidPrice getTotalBidprice() {
		BidPrice bidPrice = new BidPrice();
		bidPrice.setId("ID_" + Messages.BidPrice);
		bidPrice.setBidPriceName(Messages.BidPrice);
		bidPrice.setBidPriceType(BidPriceEnum.IS_PRICE_ITEM);
		return bidPrice;
	}
	
}
