package com.gx.obe.bpt.utils;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import com.gx.obe.bpt.entity.BidPriceTableItem;
import com.gx.obe.bpt.entity.BidPriceTableItemCollect;
import com.gx.obe.bpt.entity.BidPriceTableTitle;
import com.gx.obe.bpt.entity.BidPriceTableValue;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.util.utils.XmlUtils;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/** 
 * @Description: 解析投标报价表格xml工具类
 * @author chenxw 
 */
public class ParseBidPriceTableUtils {
	
	private static String BID_PRICE_TABLE_DETAIL_INDEX = "BidPriceTableDetailIndex.xml";
	
	
	/**
	 * @Description: 
	 * @author chenxw
	 * @param dateObject
	 * @return 
	 */
	public static Object[] getBidPriceFiles(JSONObject jsonObject) {
		return Optional.ofNullable(jsonObject)
				.map(ParseBidPriceTableUtils::getBidPriceFileListJSONArray)
				.map(t -> t.toArray())
				.orElse(null);
	}
	
	private static JSONArray getBidPriceFileListJSONArray(JSONObject jsonObject) {
		try {
			return jsonObject.getJSONArray("bidPriceFileList");
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * @Description: 
	 * @author chenxw
	 * @param dateObject
	 * @return 
	 */
	public static String getFileContentByFileName(Object[] objects, String fileName) {
		return Optional.ofNullable(objects)
				.map(Arrays::stream)
				.flatMap(s -> s
						.map(t -> (JSONObject) t)
						.filter(t -> t.getString("fileName").equals(fileName))
						.findAny())
				.map(t -> t.getString("fileContent"))
				.orElse(null);
	}
	
	/**
	 * @Description: 解析为ElementList
	 * @author chenxw
	 * @param filePath xml文件路径
	 * @return 
	 */
	public static List<?> resolverElementList(String filePath) {
		return Optional.ofNullable(filePath)
				.map(XmlUtils::getXmlDocument)
				.map(ParseBidPriceTableUtils::resolverElementList)
				.orElse(null);
	}
	
	/**
	 * @Description: 解析为ElementList
	 * @author chenxw
	 * @param filePath xml文件路径
	 * @return 
	 */
	public static List<?> resolverElementList(Document document) {
		return Optional.ofNullable(document)
				.map(Document::getRootElement)
				.map(Element::elements)
				.orElse(null);
	}
	
	/**
	 * @Description: 解析报价明细Element
	 * @author chenxw
	 * @param bidFilePath 投标文件目录
	 * @return 
	 * @throws Exception 
	 */
	public static Element resolverBidPriceTableElement(String bidFilePath) {
		return (Element) Optional.ofNullable(bidFilePath)
				.map(t -> t.concat(BID_PRICE_TABLE_DETAIL_INDEX))
				.map(ParseBidPriceTableUtils::resolverElementList)
				.map(List::stream)
				.flatMap(Stream::findFirst)
				.orElse(null);
	}
	
	/**
	 * @Description: 解析报价明细Element
	 * @author chenxw
	 * @param bidPriceFileList
	 * @return 
	 */
	public static Element resolverBidPriceTableElement(Object[] bidPriceFileList) {
		return (Element) Optional.ofNullable(bidPriceFileList)
				.map(Arrays::stream)
				.flatMap(s -> s
						.map(t -> (JSONObject) t)
						.filter(t -> t.getString("fileName").equals(BID_PRICE_TABLE_DETAIL_INDEX))
						.findAny())
				.map(t -> t.getString("fileContent"))
				.map(ParseBidPriceTableUtils::DocumentHelperParseText)
				.map(ParseBidPriceTableUtils::resolverElementList)
				.map(List::stream)
				.flatMap(Stream::findFirst)
				.orElse(null);
	}
	
	/**
	 * @Description: 
	 * @author chenxw
	 * @param bidPriceFileList
	 * @param tableFileName
	 * @return 
	 */
	public static List<?> resolverBidPriceTableTrList(Object[] bidPriceFileList, String tableFileName) {
		return Optional.ofNullable(bidPriceFileList)
				.map(Arrays::stream)
				.flatMap(s -> s
						.map(t -> (JSONObject) t)
						.filter(t -> t.getString("fileName").equals(tableFileName))
						.findAny())
				.map(t -> t.getString("fileContent"))
				.map(ParseBidPriceTableUtils::DocumentHelperParseText)
				.map(ParseBidPriceTableUtils::resolverElementList)
				.orElse(null);
	}
	
	public static Document DocumentHelperParseText(String xmlContent) {
		try {
			return DocumentHelper.parseText(xmlContent);
		} catch (DocumentException e) {}
		return null;
	}
	
	/**
	 * @Description: 解析报价明细的下级明细ElementList
	 * @author chenxw
	 * @param element
	 * @return 
	 */
	public static List<?> resolverBidPriceTableElementStream(Element element) {
		return Optional.ofNullable(element)
				.map(e -> e.element("subBidPriceTableList"))
				.map(Element::elements)
				.orElse(null);
	}
	
	/**
	 * @Description: 解析TableTitleList
	 * @author chenxw
	 * @param detailFilePath
	 * @param tenderId 
	 * @param suppilerid 
	 * @param priceTableId 
	 * @return 
	 */
	public static List<BidPriceTableTitle> resolverBidPriceTableTitleStream(
			List<?> detailTrElementList, 
			String tenderId, 
			String supplierId, 
			String priceTableId) {
		return Optional.ofNullable(detailTrElementList)
				.map(List::stream)
				.flatMap(s -> s
						.map(t -> (Element)t)
						.filter(t -> "true".equalsIgnoreCase(t.attributeValue("isTitle"))).findAny())
				.map(Element::elements)
				.map(l -> IntStream.range(0, l.size())
						.mapToObj(i -> {
							BidPriceTableTitle t = initBidPriceTableTitle((Element) l.get(i));
							t.setTenderId(tenderId);
							t.setSupplierId(supplierId);
							t.setPriceTableId(priceTableId);
							t.setSortNo(i + 1);
							return t;
						})
						.collect(Collectors.toList()))
				.get();
	}
	
	/**
	 * @Description: 解析TableItemList
	 * @author chenxw
	 * @param detailFilePath
	 * @param tenderId
	 * @param supplierId
	 * @param priceTableId
	 * @param subDetalTableId
	 * @param subCollectTableId
	 * @return 
	 */
	public static List<BidPriceTableItem> resolverBidPriceTableItemStream(
			List<?> detailTrElementList, 
			String tenderId, 
			String supplierId, 
			String priceTableId,
			Function<String, String> subDetalTableIdFunction, 
			Function<String, String> subCollectTableIdFunction, 
			ItemStreamListener listener) {
		return Optional.ofNullable(detailTrElementList)
				.map(List::stream)
				.map(s -> s
						.filter(t -> "false".equalsIgnoreCase(((Element) t).attributeValue("isTitle")))
						.collect(Collectors.toList()))
				.map(l -> IntStream.range(0, l.size())
						.mapToObj(i -> {
							Element e = (Element) l.get(i);
							BidPriceTableItem t = initBidPriceTableItem(e);
							t.setTenderId(tenderId);
							t.setSupplierId(supplierId);
							t.setPriceTableId(priceTableId);
							t.setSortNo(i + 1);
							Optional.ofNullable(listener).ifPresent(lr -> lr.apply(t.getId(), e));
							Optional.ofNullable(subDetalTableIdFunction).map(f -> f.apply(t.getSubjectTableId())).ifPresent(t::setSubDetalTableId);
							Optional.ofNullable(subCollectTableIdFunction).map(f -> f.apply(t.getSubjectTableId())).ifPresent(t::setSubCollectTableId);
							return t;
						})
						.collect(Collectors.toList()))
				.orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 解析TableValueList
	 * @author chenxw
	 * @param tdElementList
	 * @param tenderId
	 * @param supplierId
	 * @param priceTableId
	 * @param tableItemId
	 * @param tableTitleIdFunction
	 * @return 
	 */
	public static List<BidPriceTableValue> resolverBidPriceTableValueStream(
			List<?> tdElementList, 
			String tenderId, 
			String supplierId, 
			String priceTableId, 
			String tableItemId, 
			Function<Integer, String> tableTitleIdFunction) {
		return Optional.ofNullable(tdElementList)
				.map(l -> IntStream.range(0, l.size())
						.mapToObj(i -> {
							BidPriceTableValue bidPriceTableValue = initBidPriceTableValue((Element) l.get(i));
							bidPriceTableValue.setTenderId(tenderId);
							bidPriceTableValue.setSupplierId(supplierId);
							bidPriceTableValue.setPriceTableId(priceTableId);
							bidPriceTableValue.setTableItemId(tableItemId);
							bidPriceTableValue.setTableTitleId(Optional.ofNullable(tableTitleIdFunction).map(f -> f.apply(i)).orElseThrow(RuntimeException::new));
							return bidPriceTableValue;
						})
						.collect(Collectors.toList()))
				.orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 解析TableItemCollectList
	 * @author chenxw
	 * @param totalTrElementList
	 * @param tenderId 
	 * @param supplierId 
	 * @param priceTableId 
	 * @return 
	 */
	public static List<BidPriceTableItemCollect> resolverBidPriceTableItemCollectStream(
			List<?> totalTrElementList, 
			String tenderId, 
			String supplierId, 
			String priceTableId) {
		return Optional.ofNullable(totalTrElementList)
				.map(List::stream)
				.map(s -> s
						.map(t -> (Element)t)
						.map(ParseBidPriceTableUtils::initBidPriceTableItemCollect)
						.peek(t -> {
							t.setTenderId(tenderId);
							t.setSupplierId(supplierId);
							t.setPriceTableId(priceTableId);
						})
						.collect(Collectors.toList()))
				.orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 初始化TableTitle
	 * @author chenxw
	 * @param element
	 * @return 
	 */
	public static BidPriceTableTitle initBidPriceTableTitle(Element tdElement) {
		return Optional.ofNullable(tdElement).map(e -> {
			BidPriceTableTitle bidPriceTableTitle = new BidPriceTableTitle();
			bidPriceTableTitle.setId(UuidUtils.getUUID());
			bidPriceTableTitle.setTitleName(e.getStringValue());
			bidPriceTableTitle.setWidth(e.attributeValue("width"));
			bidPriceTableTitle.setColType(e.attributeValue("colType"));
			bidPriceTableTitle.setContentAlignment(e.attributeValue("contentAlignment"));
			return bidPriceTableTitle;
		}).orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 初始化TableItem
	 * @author chenxw
	 * @param trElement
	 * @return 
	 */
	public static BidPriceTableItem initBidPriceTableItem(Element trElement) {
		return Optional.ofNullable(trElement).map(e -> {
			BidPriceTableItem bidPriceTableItem = new BidPriceTableItem();
			bidPriceTableItem.setId(UuidUtils.getUUID());
			bidPriceTableItem.setRowType(e.attributeValue("rowType"));
			bidPriceTableItem.setSubjectTableId(e.attributeValue("subjectTableID"));
			return bidPriceTableItem;
		}).orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 初始化TableValue
	 * @author chenxw
	 * @param tdElement
	 * @return 
	 */
	public static BidPriceTableValue initBidPriceTableValue(Element tdElement) {
		return Optional.ofNullable(tdElement).map(e -> {
			BidPriceTableValue bidPriceTableValue = new BidPriceTableValue();
			bidPriceTableValue.setId(UuidUtils.getUUID());
			bidPriceTableValue.setValue(e.getText());
			return bidPriceTableValue;
		}).orElseThrow(RuntimeException::new);
	}
	
	/**
	 * @Description: 初始化TableItemCollect
	 * @author chenxw
	 * @param trElement
	 * @return 
	 */
	public static BidPriceTableItemCollect initBidPriceTableItemCollect(Element trElement) {
		return Optional.ofNullable(trElement).map(Element::elements).filter(l -> l.size() > 1).map(l -> {
			BidPriceTableItemCollect bidPriceTableItemCollect = new BidPriceTableItemCollect();
			bidPriceTableItemCollect.setId(UuidUtils.getUUID());
			Optional.ofNullable((Element)l.get(0)).map(Element::getText).ifPresent(bidPriceTableItemCollect::setCollectKey);
			Optional.ofNullable((Element)l.get(1)).map(Element::getText).ifPresent(bidPriceTableItemCollect::setCollectValue);
			return bidPriceTableItemCollect;
		}).orElseThrow(RuntimeException::new);
	}
	
	public interface ItemStreamListener {
		void apply(String tableItemId, Element trElement);
	}
	
}
